@@ -2,6 +2,7 @@ from .Priogrid import Priogrid
 from .ViewsMonth import ViewsMonth
 from .Country import Country
 from .scratch import fetch_ids, fetch_ids_df, cache_manager
+from .FuzzyCountry import FuzzyCountry
 import pandas as pd
 import warnings
 from functools import partial
@@ -170,6 +171,17 @@ class CAccessor:
     def month_end(self):
         return self._obj.apply(lambda row: Country(row.c_id).month_end, axis=1)
 
+    @property
+    def pg_id(self):
+        """
+        Explodes a data frame at c/cm/cy level to the corresponding pg/pgm/pgy level maintaining all values intact.
+        :return: A pg/pgm/pgy data frame containing the same data.
+        """
+        z = self._obj.copy().drop_duplicates()
+        z['pg_id'] = z.apply(lambda x: [j.id for j in Country(x.c_id).priogrids()], axis=1)
+        z = z.explode('pg_id')
+        return z
+
     @classmethod
     def from_iso(cls, df, iso_col='iso', month_col=None):
         """
@@ -364,6 +376,23 @@ class PgAccessor:
     def db_id(self):
         return self._obj
 
+    @property
+    def c_id(self):
+        """
+        Returns a column containing country corresponding to the priogrid in the current panel (at present)
+        :return: A column of ViEWS country IDs
+        """
+        return self._obj.apply(lambda row: Country.from_priogrid(row.pg_id).id, axis=1)
+
+    def c_id_at_year(self, year):
+        """
+        Returns the country corresponding to a priogrid in a given year
+        :param year: A year scalar, e.g. 1989
+        :return: A column of ViEWS country IDs
+        """
+        return self._obj.apply(lambda row: Country.from_priogrid(row.pg_id, year).id, axis=1)
+
+
     @classmethod
     def from_latlon(cls, df, lat_col='lat', lon_col='lon'):
         """
@@ -831,15 +860,15 @@ class CMAccessor(CAccessor, MAccessor):
         return structure[structure.month_id <= max_month]
 
     @classmethod
-    def new_africa(cls):
-        ids = CMAccessor.new_structure()
+    def new_africa(cls, max_month=621):
+        ids = CMAccessor.new_structure(max_month=max_month)
         africa = CAccessor.new_structure()
         africa = africa[africa.c.in_africa].c_id
         return ids[ids.c_id.isin(africa)]
 
     @classmethod
-    def new_middle_east(cls):
-        ids = CMAccessor.new_structure()
+    def new_middle_east(cls, max_month=621):
+        ids = CMAccessor.new_structure(max_month=max_month)
         me = CAccessor.new_structure()
         me = me[me.c.in_me].c_id
         return ids[ids.c_id.isin(me)]
@@ -937,6 +966,11 @@ class PGMAccessor(PgAccessor, MAccessor):
         z['valid_year_month_latlon'] = z.valid_year_month & z.valid_latlon
         return z
 
+    @property
+    def country(self):
+        raise NotImplementedError("""Due to the asymmetric format of the DB (PGM is just Africa and ME), 
+        converting between PGM and CM is not supported in ingester as data loss. Use PGY->CY instead!""")
+
     @classmethod
     def soft_validate(cls, df):
         z = df.copy()
@@ -1169,6 +1203,10 @@ class PGYAccessor(PgAccessor):
                               how='left').id
         return z
 
+    @property
+    def c_id(self):
+        return self._obj.apply(lambda row: Country.from_priogrid(row.pg_id, row.year_id).id, axis=1)
+
     def db_id(self):
         return PGYAccessor.__db_id(self._obj)
 
@@ -1183,4 +1221,41 @@ class PGYAccessor(PgAccessor):
         return z
 
 
+@pd.api.extensions.register_dataframe_accessor("fuzzy_country")
+class FuzzyCountryAccessor:
+    def __init__(self, pandas_obj):
+        self._validate(pandas_obj)
+        self._obj = pandas_obj
+        self.__precision = 0.8
 
+    @staticmethod
+    def _validate(obj):
+        pass
+
+    @property
+    def precision(self):
+        return self.__precision
+
+    @precision.setter
+    def precision(self, precision=0.8):
+        """
+        Set the minimum precision needed to match the names in the df against the list of names
+        Defined as the Jaro Winkler distance normalized as a pseudo-probability of a match.
+        :param precision: A float, 0.01-1.00 for the minimum precision needed for a match.
+        :return: Sets the property for a given df.
+        """
+        if 0 < precision < 1:
+            self.__precision  = precision
+        else:
+            warnings.warn("Precision outside [0,1] interval. Unchanged")
+
+    def isoab(self, col_name):
+        """
+        Returns the three-letter ISO code for a given column containing country names
+        Uses fuzzy matching on names, at a precision given by the precision parameter
+        default 0.8. None is returned if no name in a field
+        :param col_name: A Pandas column containing names to match
+        :return: A column of ISO codes obtained from the matches. None if no match was found.
+        """
+        z = self._obj.apply(lambda row: FuzzyCountry(row[col_name], self.__precision).isoab, axis=1)
+        return z
